local SDA_Version = "1.0" -- Current addon version
local SDA_Debug = false -- False will actually delete items, True only shows what is found
local SDA_Deleted = {} -- Keep a history of deleted items for the log command

-- CONSTANTS
local GREY = "|cff999999"
local CYAN = "|cff33ccff"
local YELLOW = "|cffffff00"
local RED = "|cffff0000"
local WHITE = "|cffffffff"
local GREEN = "|cff1eff00"
local PURPLE = "|cff9f3fff"
local BLUE = "|cff0070dd"
local ORANGE = "|cffff8400"
local SDA_ITEM = 0
local SDA_JUNK = 1
local SDA_Syntax = YELLOW.."/sda item ITEM_NAME or /sda junk"

-- PARSES COMMAND-LINE PARAM AND CALLS PROPER FUNCTIONS
function SDA_GetCmd(cmd)
	if cmd then
		local a,b,c = string.find(cmd,"^(%S+)")
		if c == "item" and # cmd ~= # c then
			SDA_Delete(SDA_ITEM, string.trim(string.sub(cmd, b+2)))
		elseif c == "junk" then
			SDA_Delete(SDA_JUNK)
		elseif c == "log" then
			SDA_ShowDeleted()
		elseif c == "debug" then
			SDA_ToggleDebug()
		else
			SDA_Print(SDA_Syntax)
		end
	else
		SDA_Print(SDA_Syntax);
	end

end

-- EXTRACTS INFO FROM ITEMLINK STRING
function SDA_ParseItemLink(itemLink)

	-- Regex pattern to use
	itemLink = string.trim(itemLink)
	local REGEX = "^%|(%x+)%|H(item:(%d+):.+)%|h%[(.+)]%|h%|r$"
	local st,nd,itemColor,itemString,itemID,itemName = string.find(itemLink,REGEX)

	return itemName,itemID,itemString,itemColor

end

-- DELETES ALL REQUESTED ITEMS
function SDA_Delete(delType,cmdParam)

	-- DELETE ALL ITEMS BY NAME
	if delType == SDA_ITEM then

		if cmdParam then
			-- Get if cmdParam is an itemLink get itemName
			local itemName = SDA_ParseItemLink(cmdParam)

			if itemName == nil then
				itemName = cmdParam
			end
			SDA_PrintDebug("itemName = '"..itemName.."'")

			-- Get total count of item in all bags and only get in loop
			-- if there's at least 1. This is only to avoid looping for nothing
			local itemCount = GetItemCount(itemName)
			if itemCount > 0 then

				local deletedCount = 0

				-- Loop through every bag
				for bag = 0,4,1 do
					-- Loop through every slot
					for slot = 1, 32, 1 do

						-- If slot is not empty...
						if GetContainerItemLink(bag,slot) ~= nil then

							-- Get item link and count
							local itemLink = GetContainerItemLink(bag,slot)
							local slotItemName = SDA_ParseItemLink(itemLink)
							local _, slotItemCount = GetContainerItemInfo(bag,slot)

							-- If item link matches exactly item requested, delete it
							if (slotItemName == itemName) then
								SDA_PrintDebug(ORANGE.."Found "..WHITE..slotItemCount.."x "..itemLink)

								-- Actually delete only if not in debug mode
								if SDA_Debug == false then
									PickupContainerItem(bag,slot)
									DeleteCursorItem()

									-- Increment counter for deleted count
									deletedCount = deletedCount + slotItemCount
								end
							end

						end

					end
				end

				-- Must get itemLink again since the previous one was local to the for loop
				local _,itemLink = GetItemInfo(itemName)

				-- If something was actually deleted, print it
				if deletedCount > 0 then
					SDA_Print(RED.."Deleted "..WHITE..itemCount.."x "..itemLink)

					-- Increment the deleted counter for that item in the deleted history
					if SDA_Deleted[itemLink] then
						SDA_Deleted[itemLink] = SDA_Deleted[itemLink] + deletedCount
					else
						SDA_Deleted[itemLink] = deletedCount
					end
				end

			end
		else
			SDA_Print(SDA_Syntax)
		end

	-- DELETE ALL JUNK
	elseif delType == SDA_JUNK then

		-- Loop through every bag
		for bag = 0,4,1 do
			-- Loop through every slot
			for slot = 1, 32, 1 do

				-- If slot is not empty...
				if GetContainerItemLink(bag,slot) ~= nil then

					-- Get item link, count and quality
					local itemLink = GetContainerItemLink(bag,slot)
					local _, itemCount = GetContainerItemInfo(bag,slot)
					local itemName, _, itemRarity = GetItemInfo(itemLink)

					-- Only delete it itemRarity is 0 (junk)
					if itemRarity == 0 then
						SDA_PrintDebug(ORANGE.."Found "..WHITE..itemCount.."x "..itemLink)

						-- Actually delete only if not in debug mode
						if SDA_Debug == false then
							PickupContainerItem(bag,slot)
							DeleteCursorItem()

							-- If something was actually deleted, print it
							SDA_Print(RED.."Deleted "..WHITE..itemCount.."x "..itemLink)

							-- Increment counter for deleted count
							if SDA_Deleted[itemLink] then
								SDA_Deleted[itemLink] = SDA_Deleted[itemLink] + itemCount
							else
								SDA_Deleted[itemLink] = itemCount
							end
						end

					end

				end

			end
		end

	end
end

-- PRINT A LOG OF ALL DELETED ITEMS DURING CURRENT SESSION
function SDA_ShowDeleted()

	local count = 0
	table.sort(SDA_Deleted)
	for i,v in pairs(SDA_Deleted) do
		count = count + 1
	end

	if count > 0 then
		SDA_Print(" ")
		SDA_Print(WHITE.."--------------------------------------")
		SDA_Print(GREEN.."Items deleted in current session:")
		SDA_Print(WHITE.."--------------------------------------")
		totalDeleted = 0
		for i,n in pairs(SDA_Deleted) do
			totalDeleted = totalDeleted + n
			SDA_Print("  "..WHITE..n.."x "..i)
		end
		SDA_Print(WHITE.."--------------------------------------")
		SDA_Print(GREEN.."Total: "..WHITE..totalDeleted.." items deleted")
		SDA_Print(" ")
	else
		SDA_Print(GREEN.."Nothing has been deleted in this session.")
	end
end

-- PRINTS STRING TO DEFAULT CHAT FRAME
function SDA_Print(message)
	DEFAULT_CHAT_FRAME:AddMessage(message)
end

-- PRINTS DEBUG MESSAGE TO DEFAULT CHAT FRAME
-- (Only if SDA_Debug = true)
function SDA_PrintDebug(message)
	if SDA_Debug == true then
		SDA_Print(GREEN.."SDA Debug: "..message)
	end
end

-- TOGGLES DEBUG MODE ON AND OFF
function SDA_ToggleDebug()
	if SDA_Debug == true then
		SDA_Print(YELLOW.."SDA Debug Mode ["..RED.."Disabled"..YELLOW.."]")
		SDA_Debug = false
	else
		SDA_Print(YELLOW.."SDA Debug Mode ["..GREEN.."Enabled"..YELLOW.."]")
		SDA_Debug = true
	end
end

-- SLASH COMMAND DECLARATION
SLASH_SDA_CMD1 = "/sda"
SlashCmdList["SDA_CMD"] = SDA_GetCmd

-- PRINTS LOADED MESSAGE
SDA_Print("SlashDeleteAll v."..SDA_Version.." loaded.")
